/*
 * DCAP - Direct CAPture
 * Copyright (C) 2020 Oliver Kirsebom
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <TF1.h>
#include <TMath.h>
#include "Math/WrappedTF1.h"
#include "Math/BrentRootFinder.h" 
#include <DCAP/FinalWaveFunc.h>
#include <DCAP/LogDerivDiff.h>

using namespace std;
using namespace ORM;
using namespace DCAP;


finalWaveFunc::finalWaveFunc(shared_ptr<ParticleChannel>& channel, double energy, double rMax, double eMin, double eMax) 
: channel(channel), E(energy), eps(0.001) {
    V0 = computeSquareWellDepth(channel, energy, eMin, eMax);
    A = 1;
    B = channel->regSphericalBesselFunction(E + V0) / (channel->whittakerFunction(-E) / channel->getChannelRadius());
    A = computeNorm(rMax);
}

double finalWaveFunc::operator() (double r) {
    double psi;
    if (r > channel->getChannelRadius()) psi = B * channel->whittakerFunction(-E, r) / r; //outer w.f.
    else psi = channel->regSphericalBesselFunction(E + V0, r); //inner w.f.
    return A * psi;
}

double finalWaveFunc::getSquareWellDepth() const {
    return V0;
}

double finalWaveFunc::computeNorm(double rMax) {
    TF1 f{"", [&](double* x, double* p) { //integrand
        double r = x[0];
        double u = (*this)(r) * r;
        return pow(u, 2.);
    },0,0,0};
    return sqrt(1. / f.Integral(eps, rMax));
}

double DCAP::computeSquareWellDepth(shared_ptr<ParticleChannel>& channel, double energy, 
    double energyMin, double energyMax) {
    double r = channel->getChannelRadius();
    LogDerivDiff fobj(channel, energy, r);
    TF1 * f = new TF1("f", fobj, energyMin-10, energyMax+10, 0);
    ROOT::Math::WrappedTF1 wf(*f);
    ROOT::Math::BrentRootFinder brf;
    brf.SetFunction(wf, energyMin, energyMax);
    brf.Solve();
    return brf.Root();
}


